@@ -1,6 +1,5 @@
 library(grid)
 postscript("notes.ps")
-grid.start()
 grid.rect(x=unit(1, "inches"), y=unit(.1, "npc"),
           width=unit(.5, "native"), height=unit(1, "lines"),
           just=c("left", "bottom"))
@@ -16,14 +15,14 @@ gt <- grid.text("howdy")
 grid.rect(width=unit(1, "grobwidth", gt), height=unit(1, "lines"))
 
 grid.newpage()
-gf <- grid.frame()
+gf <- grid.frame(draw=TRUE)
 grid.pack(gf, grid.rect(x=unit(1, "npc"), 
                         w=unit(1, "char"), h=unit(1, "char"),
                         just=c("right", "centre"), draw=F))
 grid.pack(gf, grid.text("A label", draw=F), side="right")
 
 grid.newpage()
-gf <- grid.frame()
+gf <- grid.frame(draw=TRUE)
 grid.pack(gf, grid.rect(width=unit(1, "inches"),
                         height=unit(.5, "npc"),
                         draw=F))
